'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.validateState = exports.validateField = undefined;

var _mapValues2 = require('lodash/mapValues');

var _mapValues3 = _interopRequireDefault(_mapValues2);

var _find2 = require('lodash/find');

var _find3 = _interopRequireDefault(_find2);

var _without2 = require('lodash/without');

var _without3 = _interopRequireDefault(_without2);

var _forEach2 = require('lodash/forEach');

var _forEach3 = _interopRequireDefault(_forEach2);

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _Validators = require('./Validators');

var _Validators2 = _interopRequireDefault(_Validators);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* Utility that splits a string up first by the
 * character '|' and then further more by the
 * character ':'.
 * */
var splitValidationRule = function splitValidationRule(rule, cb) {
	rule = rule || "";
	(0, _forEach3.default)(rule.split("|"), function (splitRule) {
		splitRule = splitRule.split(":");
		return cb(splitRule[0], (0, _without3.default)(splitRule, splitRule[0]));
	});
};

/* Given a Field, validate it based on its local
 * validation string.
 * */
var validateField = exports.validateField = function validateField(field) {
	var validators = _extends({}, _Validators2.default, field.validators);

	var nextFieldState = _extends({}, field, {
		valid: true,
		error: null
	});

	splitValidationRule(field.validation, function (name, args) {
		var validator = (0, _find3.default)(validators, function (validator, key) {
			return key == name;
		});

		if (validator && field.value && field.value != "") {
			var res = validator({
				value: field.value,
				args: args
			});

			nextFieldState = _extends({}, field, {
				error: null
			}, res);

			return res.valid;
		}
	});

	if (field.pattern) {
		if (!field.pattern.test(field.value)) {
			nextFieldState = _extends({}, field, {
				valid: false,
				error: field.patternError || "Validation failed"
			});
		}
	}

	return nextFieldState;
};

/* Given a Field, validate it based on its global
 * validation string. This validation property is
 * not used internally by Flum.
 * */
var validateState = exports.validateState = function validateState(state) {
	var valid = true;

	var validate = function validate(field) {
		var validators = _extends({}, _Validators2.default, field.validators);

		var localValidatedField = validateField(field);
		var globalValidatedField = _extends({}, localValidatedField);
		valid = !valid ? valid : localValidatedField.valid;

		splitValidationRule(field.postValidation, function (name, args) {
			var validator = (0, _find3.default)(validators, function (validator, key) {
				return key == name;
			});

			if (validator) {
				var res = validator({
					value: field.value,
					state: state,
					args: args
				});

				globalValidatedField = _extends({}, localValidatedField, {
					valid: res.valid ? localValidatedField.valid : false
				});
				globalValidatedField.error = res.error ? res.error : !globalValidatedField.valid ? localValidatedField.error : null;

				valid = !valid ? valid : globalValidatedField.valid;

				return res.valid;
			}
		});

		return globalValidatedField;
	};

	var mapAndValidate = function mapAndValidate(state) {
		return (0, _mapValues3.default)(state, function (data) {
			if ((typeof data === 'undefined' ? 'undefined' : _typeof(data)) !== 'object') return data;
			if (data.__flum) return validate(data);
			return mapAndValidate(data);
		});
	};
	var nextState = mapAndValidate(state);

	return {
		valid: valid,
		state: nextState
	};
};